/******************************************************************************
**
**	Apple Macintosh Developer Technical Support
**
**	Sessions is a System 7.0 and Communications Toolbox aware sample
**
**	Copyright  1989 Apple Computer, Inc.
**	All rights reserved.
**
**	Author:		James Beninghaus
**	Versions:	1.00				1/91
**
**	Sessions provides simple terminal and file transfer services. 
**	This application requires System 7 to run. 
**	Gestalt is used to test the operating environment.
**	The CTB Connection, File Transfer and Terminal Managers are used
**	extensively.
**	The new File System and Resource Manager calls are used to save and
**	restore document and preference files.
**	Aliases are used to track open documents.
**	The Folder Manager is used to create a preference file in the Preference
**	folder and temporary files in the Temorary Folder.
**	The new Standard File Package routines are called to select documents to
**	open and files to send during a File Transfer operation. 
**
******************************************************************************/

#include	<Sessions.h>

pascal OSErr CoerceAliasToFSS(AEDesc *desc, DescType toType, long handlerRefcon, AEDesc *result) {
	auto	OSErr		osErr;
	auto	FSSpec		file;
	auto	Boolean		wasChanged;

	if (noErr == (osErr = ResolveAlias(nil, (AliasHandle) (*desc).dataHandle, &file, &wasChanged)))
		osErr = AECreateDesc(typeFSS, (Ptr) &file, sizeof(FSSpec), result);
		
	return osErr;
}


pascal OSErr OpenApplication (AppleEvent *message, AppleEvent *reply, long refcon) { 
	auto	OSErr		osErr;
	auto	SessionPtr	session;

	/* Open a default Session as according to IM and CK */
	if (noErr == (osErr = DoPreferedSession(&session)))
		InsertSession(session);
	else {
		ErrorAlert(osErr, "\pA default session is being created");
		(void) DoCloseSession(session);
		if (noErr == (osErr = DoNewSession(&session)))
			InsertSession(session);
		else {
			ErrorAlert(osErr, "\pNo session could be created");
			(void) DoCloseSession(session);
		}
	}

	return noErr;
}

pascal OSErr OpenDocuments (AppleEvent *message, AppleEvent *reply, long refcon) {
	auto	OSErr				osErr;
	auto	AEDesc				directObject;
	auto	FSSpec				file;
	auto	short				n;
	auto	long				numFilesToOpen;
	auto	AEKeyword			ignoredKeyWord;
	auto	DescType			ignoredType;
	auto	Size				ignoredSize;
	auto	FInfo				fndrInfo;
	auto	SessionPtr			session;
	auto	CMBufferSizes		size;
	auto	CMStatFlags			status;
		
	if (noErr == (osErr = AEGetParamDesc(message, keyDirectObject, typeAEList, &directObject))) {
		if (noErr == (osErr = AECountItems(&directObject, &numFilesToOpen)))
			for (n = 1; n <= numFilesToOpen; ++n) {
				if (noErr == (osErr = AEGetNthPtr(&directObject, n, typeFSS, &ignoredKeyWord, &ignoredType, (Ptr) &file, sizeof(file), &ignoredSize))) {
					if (noErr == (osErr = FSpGetFInfo(&file, &fndrInfo)))
						switch(fndrInfo.fdType) {
							case kDocFileType :
								if (noErr == (osErr = DoOpenSession(&session, &file, 0 != (isStationary & fndrInfo.fdFlags))))
									InsertSession(session);
								else
									(void) DoCloseSession(session);
								break;
							default :
								/* a Hack to demonstrate auto file transfers */
								if (noErr == (osErr = DoPreferedSession(&session))) {
									InsertSession(session);
									if (noErr == (osErr = ConnectionConnect((*session).connection))) {
										do {
											SessionIdle(session);
											CMStatus((*session).connection, size, &status);
										} while (0 == (status & cmStatusOpen));
										osErr = FileTransferSend((*session).fileTransfer, &file);
									}
								}

								break;
						}
				} else 
					break;
			}
		osErr = AEDisposeDesc(&directObject);
	}

	return osErr;
}

pascal OSErr PrintDocuments (AppleEvent *message, AppleEvent *reply, long refcon) { 
	return noErr;
}

pascal OSErr QuitApplication (AppleEvent *message, AppleEvent *reply, long refcon) { 
	gProcessing = false; 
	return noErr;
}


OSErr InitAppleEvents() {
	auto	OSErr	osErr;
	
	if (noErr == (osErr = AEInstallEventHandler(kCoreEventClass,	kAEOpenApplication,	(ProcPtr) OpenApplication,	0,	false)))
		if (noErr == (osErr = AEInstallEventHandler(kCoreEventClass,	kAEOpenDocuments,	(ProcPtr) OpenDocuments,	0,	false)))
			if (noErr == (osErr = AEInstallEventHandler(kCoreEventClass,	kAEPrintDocuments,	(ProcPtr) PrintDocuments,	0,	false)))
				if (noErr == (osErr = AEInstallEventHandler(kCoreEventClass,	kAEQuitApplication,	(ProcPtr) QuitApplication,	0,	false)))

					if (noErr == (osErr = AEInstallCoercionHandler(typeAlias, typeFSS, (ProcPtr) CoerceAliasToFSS, 0, true, false)))

						osErr = noErr;
	return osErr;
}

Boolean CMWindow(ConnHandle connection) {
	auto	SessionPtr	session;
	
	session = gWorkSessionList;
	while (nil != session)
		if ((*session).connection == connection)
			return true;
		else
			session = (*session).nextSession;
	return false;
}

Boolean TMWindow(TermHandle terminal) {
	auto	SessionPtr	session;

	session = gWorkSessionList;
	while (nil != session)
		if ((*session).terminal == terminal)
			return true;
		else
			session = (*session).nextSession;
	return false;
}

Boolean FTWindow(FTHandle fileTransfer) {
	auto	SessionPtr	session;

	session = gWorkSessionList;
	while (nil != session)
		if ((*session).fileTransfer == fileTransfer)
			return true;
		else
			session = (*session).nextSession;
	return false;
}

Boolean	ToolEvent(EventRecord *event, long refCon) {
	auto	Boolean		toolEvent	= false;
		
	if (toolEvent = CMWindow((ConnHandle) refCon))
		CMEvent((ConnHandle) refCon, event);
	else if (toolEvent = TMWindow((TermHandle) refCon))
		TMEvent((TermHandle) refCon, event);
	else if (toolEvent = FTWindow((FTHandle) refCon))
		FTEvent((FTHandle) refCon, event);
				
	return toolEvent;
}


void EventLoop() {
	auto	OSErr				osErr;
	auto	Boolean				gotEvent;
	auto	EventRecord			event;
	auto	long				menuItem;
	auto	WindowPtr			window;
	auto	SessionPtr			session;
	auto	long				refCon;
	auto	Boolean				toolEvent;
	auto	short				part;
	auto	CMBufferSizes		size;
	auto	CMStatFlags			status;
	auto	Rect				rect;
	auto	long				limit;
	auto	char				key;
	auto	MenuHandle			menu;
	
	do {
		
		osErr		= noErr;
		toolEvent	= false;
		window		= nil;
		
		SetCursor(&qd.arrow);
		
		event.what = -1;

		gotEvent = WaitNextEvent(everyEvent, &event, 0, nil);
		
		switch (event.what) {
			case mouseDown:
				part = FindWindow(event.where, &window);
				switch (part) {
					case inMenuBar:
						window = FrontWindow();			
						break;
					case inSysWindow:
					case inContent :
					case inGrow :
					case inDrag :
					case inGoAway :
						break;
				}
				break;
			case autoKey :
			case keyDown :
				window = FrontWindow();
				break;
			case updateEvt :
				window = (WindowPtr) event.message;
				break;
			case activateEvt :
				window = (WindowPtr) event.message;
				break;
			case nullEvent :
			case mouseUp :
			case keyUp :
			case diskEvt :
			case networkEvt :
			case driverEvt :
			case app1Evt :
			case app2Evt :
			case app3Evt :
			case app4Evt :
				break;
		}
		
		
		if (nil != window) {
			if (0 != (refCon = GetWRefCon(window)))
				toolEvent = ToolEvent(&event, refCon);
		}

		if (! toolEvent) {
			switch (event.what) {
				case nullEvent:
					session = gWorkSessionList;
					while (nil != session)
						session = noErr == (osErr = SessionIdle(session)) ? (*session).nextSession : nil;
					break;
				case mouseDown:
					part = FindWindow(event.where, &window);
					switch (part) {
						case inMenuBar:
							window = FrontWindow();			
							if (noErr == (osErr = MenuAdjust(window))) {					
								menuItem = MenuSelect(event.where);
								osErr = MenuCommand(HiWord(menuItem), LoWord(menuItem), window);
							}
							break;
						case inSysWindow:
							SystemClick(&event, window); // do we really need this in System 7 ?
							break;
						case inContent :
							if (! (*(WindowPeek) window).hilited) {
								if (FrontWindow() != gSplash)
									SelectWindow(window);
							} else if (session = IsSessionWindow(window)) {
								TMClick((*session).terminal, &event);
							} else if (IsAboutWindow(window)) {
								DisposDialog(gSplash);
								gSplash = nil;
								HMSetMenuResID(mApple, -1);
								if (nil != (menu = GetMHandle(mApple)))
									EnableItem(menu, 0);
								HMSetMenuResID(mFile, -1);
								if (nil != (menu = GetMHandle(mFile)))
									EnableItem(menu, 0);
								HMSetMenuResID(mEdit, -1);
								if (nil != (menu = GetMHandle(mEdit)))
									EnableItem(menu, 0);
								HMSetMenuResID(mSession, -1);
								if (nil != (menu = GetMHandle(mSession)))
									EnableItem(menu, 0);
								HMSetMenuResID(mSettings, -1);
								if (nil != (menu = GetMHandle(mSettings)))
									EnableItem(menu, 0);
							//	if (noErr == HMGetHelpMenuHandle(&menu))
							//		EnableItem(menu, 1);
								DrawMenuBar();
							}
							break;
						case inGrow :
							if (session = IsSessionWindow(window)) {
								rect = qd.screenBits.bounds;
								InsetRect(&rect, 8, 8);
								rect.top = 64;
								rect.left = 128;
								limit = GrowWindow(window, event.where, &rect);
								if (0 != limit) {
									SizeWindow(window, LoWord(limit), HiWord(limit), true);
									TMResize((*session).terminal, &(*session).window.port.portRect);
								}
							}
							break;
						case inDrag :
							if (IsAboutWindow(window))
								DragWindow(window, event.where, &qd.screenBits.bounds);
							else if (FrontWindow() != gSplash)
								DragWindow(window, event.where, &qd.screenBits.bounds);
							break;
						case inGoAway :
							if (IsDescAccWindow(window))
								CloseDeskAcc((*(WindowPeek) window).windowKind);
						#if 0 // because there has been a problem with the following code  it was pulled out
							else if (session = IsSessionWindow(window)) {
								DeleteSession(session);
								osErr = DoCloseSession(session);
							}
						#endif
							break;
					}
					break;
				case activateEvt :
					window = (WindowPtr) event.message;
					SetPort(window);
					if (session = IsSessionWindow(window)) {
						CMActivate((*session).connection,	(event.modifiers & activeFlag) != 0);
						TMActivate((*session).terminal,		(event.modifiers & activeFlag) != 0);
						FTActivate((*session).fileTransfer,	(event.modifiers & activeFlag) != 0);
					}
					break;
				case updateEvt :
					window = (WindowPtr) event.message;
					SetPort(window);
					BeginUpdate(window);
						if (session = IsSessionWindow(window)) 
							TMUpdate((*session).terminal, (*session).window.port.visRgn);
						else if (IsAboutWindow(window)) 
							DrawDialog((DialogPtr) window);
					EndUpdate(window);
					break;
				case keyDown :
				case autoKey :
					window = FrontWindow();
					key = event.message & charCodeMask;
					if ( event.modifiers & cmdKey ) {
						if (noErr == (osErr = MenuAdjust(window))) {					
							menuItem = MenuKey(key);
							osErr = MenuCommand(HiWord(menuItem), LoWord(menuItem), window);
						}
					} else {
						if (session = IsSessionWindow(window)) {
							if (noErr == (osErr = CMStatus((*session).connection, size, &status)))
								if (status & cmStatusOpen)
									if (! (*session).ftStart && ! (*session).ftInProgress)
										TMKey((*session).terminal, &event);
						}
					}
					break;
				case osEvt:
					switch ((event.message >> 24) & 0x0FF) {
						case kSuspendResumeMessage:
							session = gWorkSessionList;
							while (nil != session) {
								CMResume((*session).connection,		(event.message & kResumeFlag) != 0);
								TMResume((*session).terminal,		(event.message & kResumeFlag) != 0);
								FTResume((*session).fileTransfer,	(event.message & kResumeFlag) != 0);
								session = (*session).nextSession;
							}
							break;
					}
					break;
				case kHighLevelEvent:
					osErr = AEProcessAppleEvent(&event);
					break;
			}
		}
		osErr = ErrorAlert(osErr, "\pYour action could not be processed successfully");
	} while (gProcessing);
}

