/******************************************************************************
**
**	Apple Macintosh Developer Technical Support
**
**	Sessions is a System 7.0 and Communications Toolbox aware sample
**
**	Copyright  1989 Apple Computer, Inc.
**	All rights reserved.
**
**	Author:		James Beninghaus
**	Versions:	1.00				1/91
**
**	Sessions provides simple terminal and file transfer services. 
**	This application requires System 7 to run. 
**	Gestalt is used to test the operating environment.
**	The CTB Connection, File Transfer and Terminal Managers are used
**	extensively.
**	The new File System and Resource Manager calls are used to save and
**	restore document and preference files.
**	Aliases are used to track open documents.
**	The Folder Manager is used to create a preference file in the Preference
**	folder and temporary files in the Temorary Folder.
**	The new Standard File Package routines are called to select documents to
**	open and files to send during a File Transfer operation. 
**
**	NOTE: This code was compiled and linked using MPW 3.2 which supports Gestalt
**			with glue code if the Gestalt trap is not available.
**
******************************************************************************/

#define		_Storage_	true
#include	<Sessions.h>

extern void _DataInit();

/*
**	GestaltCTB is a utility routine to make sure the CTB Managers are installed.
*/

static	OSErr	GestaltCTB() {
	auto		OSErr			osErr;
	auto		long			CTBVersion;
	auto		long			CRMAttr;
	auto		long			CMAttr;
	auto		long			TMAttr;
	auto		long			FTAttr;
	
	if (noErr == (osErr = Gestalt(gestaltCTBVersion, &CTBVersion)))
		if (0x101 > CTBVersion)
			osErr = noErr;
		else 
			if (noErr == (osErr = Gestalt(gestaltCRMAttr, &CRMAttr)))
				if (noErr == (osErr = Gestalt(gestaltConnMgrAttr, &CMAttr)))
					if (noErr == (osErr = Gestalt(gestaltFXfrMgrAttr, &FTAttr)))
						if (noErr == (osErr = Gestalt(gestaltTermMgrAttr, &TMAttr)))
							if (	(CRMAttr & (1 << gestaltCRMPresent)) 		&& 
									(CMAttr  & (1 << gestaltConnMgrPresent))	&&  
									(FTAttr  & (1 << gestaltFXfrMgrPresent))	&&
									(TMAttr  & (1 << gestaltTermMgrPresent))	)
								osErr = noErr;
							else
								osErr = kUnknownError;
	return osErr;
}

/*
** InitCTB is a utility to initalize the CTB Managers.
*/

static	OSErr	InitCTB() {
	auto		OSErr			osErr;
	
	if (noErr == (osErr = InitCTBUtilities()))
		if (noErr == (osErr = InitCRM()))
			if (noErr == (osErr = InitCM()))
				if (noErr == (osErr = InitTM()))
					if (noErr == (osErr = InitFT()))
						osErr = noErr;
	return osErr;
}

/*
** SessionInit must be called during the initialization of the application.
*/

OSErr	InitSessions() {
	auto		OSErr			osErr;

	SetPt(&gChooseWhere, 10, 30);
	
	if (noErr == (osErr = GestaltCTB()))
		if (noErr == (osErr = InitCTB()))
			osErr = noErr;
	return  osErr;
}


#pragma segment Main
main() {
	auto	OSErr			osErr;
	auto	SessionPtr		session;
	auto	Handle			menuBar;
	auto	EventRecord		event;
	auto	long			i;
	
	UnloadSeg((Ptr) _DataInit);
	MaxApplZone();	

	InitGraf((Ptr) &qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
	
	/* Show splash screen */
	EventAvail(everyEvent, &event);
	EventAvail(everyEvent, &event);
	EventAvail(everyEvent, &event);
	gSplash = GetNewDialog(128, 0, (WindowPtr) -1);
	DrawDialog(gSplash);
	
	/* Initialize Communications Toolbox */
	if (noErr != (osErr = InitSessions()))
		AbortAlert(osErr, "\pThe Communicatons Toolbox could not be initalized.");
	
	/* Initialize AppleEvent stuff */
	if (noErr != (osErr = InitAppleEvents()))
		AbortAlert(osErr, "\pThe AppleEvent Handlers could not be initalized.");

	/* Setup Menubar */
	menuBar = GetNewMBar(mMenuBar);
	if (nil == menuBar)
		AbortAlert(kUnknownError, "\pThe menu bar could not be initalized.");
	SetMenuBar(menuBar);
	DisposHandle(menuBar);
	AddResMenu(GetMHandle(mApple), 'DRVR');
	DrawMenuBar();
	
	/* Setup Available Session List */
	gAvailSessionList = nil;
	for (i = 0; i < kMaxSession; ++i)
		if (nil == (session = (SessionPtr) NewPtrClear(sizeof(Session))))
			AbortAlert(osErr = MemError(), "\pSession memory could not be initalized.");
		else
			FreeSession(session);
	
	/* Setup Working Session List */
	gWorkSessionList = nil;
	
			
	DisposDialog(gSplash);
	gSplash = nil;

	gProcessing = true;
	EventLoop();
}
