/******************************************************************************
**
**	Apple Macintosh Developer Technical Support
**
**	Sessions is a System 7.0 and Communications Toolbox aware sample
**
**	Copyright  1989 Apple Computer, Inc.
**	All rights reserved.
**
**	Author:		James Beninghaus
**	Versions:	1.00				1/91
**
**	Sessions provides simple terminal and file transfer services. 
**	This application requires System 7 to run. 
**	Gestalt is used to test the operating environment.
**	The CTB Connection, File Transfer and Terminal Managers are used
**	extensively.
**	The new File System and Resource Manager calls are used to save and
**	restore document and preference files.
**	Aliases are used to track open documents.
**	The Folder Manager is used to create a preference file in the Preference
**	folder and temporary files in the Temorary Folder.
**	The new Standard File Package routines are called to select documents to
**	open and files to send during a File Transfer operation. 
**
******************************************************************************/

#include 	<Sessions.h>

OSErr PtrToHandle(Ptr p, Size n, Handle *h) {
	auto		OSErr		osErr		= noErr;
	
	if (nil == (*h = NewHandle(n)))
		osErr = MemError();
	else {
		HLock(*h);
		BlockMove(p, **h, GetHandleSize(*h));
		HUnlock(*h);
	}
	return osErr;
}

OSErr AddResourcePtr(Ptr p, Size n, ResType type, short ID, Str255 name) {
	auto		OSErr		osErr;
	auto		Handle		h;

	if (noErr == (osErr = PtrToHandle(p, n, &h))) {
		AddResource(h, type, ID, name);
		osErr = ResError();
		ReleaseResource(h);
	}
	return osErr;
}


/*
** FindPreferenceFile opens the preference file.
** If the file is missing from the preference folder then one is created.
*/

OSErr	FindPreferenceFile(FSSpecPtr prefFSSpec) {
	auto		OSErr			osErr;
	auto		short			prefVRefNum;
	auto		long			prefDirID;
	auto		short			prefRefNum;
	auto		Session			session;

	if (noErr == (osErr = FindFolder(kOnSystemDisk, kPreferencesFolderType, kCreateFolder, &prefVRefNum, &prefDirID))) {
		osErr = FSMakeFSSpec(prefVRefNum, prefDirID, kPrefFileName, prefFSSpec);
		if (fnfErr == osErr) {
			FSpCreateResFile(prefFSSpec, kCreatorType, kPrefFileType, kScriptType);
			if (noErr == (osErr = ResError())) {
				prefRefNum = FSpOpenResFile(prefFSSpec, fsRdWrPerm);
				if (-1 == prefRefNum)
					osErr = ResError();
				else {
					if (noErr == (osErr = SessionNew(&session))) {
						osErr = SessionSave(&session, prefRefNum);
						SessionClose(&session);
					}
					CloseResFile(prefRefNum);
				}
			}
		}
	}
	return osErr;
}

SessionPtr FrontSession(void) {
	auto	WindowPeek		window;
	
	window = (WindowPeek) FrontWindow();
	while (window)
		if (kSessionWindowKind == (*window).windowKind)
			break;
		else
			window = (*window).nextWindow;
	return (SessionPtr) window;
}


OSErr	EmulationWindow(WindowPtr window) {
	auto	OSErr			osErr		= noErr;

	if (nil == (GetNewWindow(rSessionWIND, (Ptr) window,  (WindowPtr) (nil == gSplash ? -1 : gSplash))))
		osErr = kUnknownError;
	else
		(*(WindowPeek)window).windowKind = kSessionWindowKind;
	return osErr;
}

/* AVAILABLE LIST UTILITIES */
SessionPtr	AvailSession() {
	auto	SessionPtr		session	= nil;
	auto	Ptr				p;
	
	if (nil != gAvailSessionList) {
		session = gAvailSessionList;
		gAvailSessionList = (*session).nextSession;
		for (p = (Ptr) session; p < (((Ptr) session) + sizeof(Session)); ++p)
			(*p) = 0;
	}
	return session;
}

void FreeSession(SessionPtr session) {
	if (session) {
		(*session).nextSession = gAvailSessionList;
		gAvailSessionList = session;
	}
}

/* WORKING LIST UTILITIES */
void InsertSession(SessionPtr session) {
	(*session).nextSession = gWorkSessionList;
	gWorkSessionList = session;
}

void DeleteSession(SessionPtr session) {
	auto	SessionPtr			p;

	if (gWorkSessionList == session)
		gWorkSessionList = (*session).nextSession;
	else {
		p = gWorkSessionList;
		while ((*p).nextSession != session);
			p = (*p).nextSession;
		(*p).nextSession = (*session).nextSession;
	}
}