#include <Sessions.h>

OSErr	ConnectionNew(ConnHandle *connection, long refCon) {
	auto	OSErr			osErr		= noErr;
	auto	short			procID;
	auto	Str255			toolName;
	auto	CMBufferSizes	sizes		= { 0, 0, 0, 0, 0, 0, 0, 0 };

	if (noErr == (osErr = CRMGetIndToolName(classCM, 1, toolName))) { 
		osErr = kUnknownError;
		procID = CMGetProcID(toolName);
		if (-1 != procID)
			if (nil == (*connection = CMNew(procID, cmData, sizes, refCon, 0))) 
				osErr = ResError();
			else
				osErr = noErr;
	}
	return osErr;
}

OSErr	ConnectionOpen(ConnHandle *connection, long refCon) {
	auto		OSErr			osErr		= noErr;
	register	Handle			h;
	register	short			procID;
	auto		CMBufferSizes	sizes		= { 0, 0, 0, 0, 0, 0, 0, 0 };

	if (nil == (h = Get1Resource(kPascalText, rCMToolName)))
		osErr = ResError();
	else {
		HLock(h);
		procID = CMGetProcID(*h);
		HUnlock(h);
		ReleaseResource(h);
		if (-1 == procID)
			osErr = cmGenericError;
		else {
			if (nil == (*connection = CMNew(procID, cmData, sizes, refCon, 0))) 
				osErr = ResError();
			else {
				if (nil == (h = Get1Resource(kCText, rCMConfigStr)))
					osErr = ResError();
				else {
					HLock(h);
					osErr = CMSetConfig(*connection, *h);
					HUnlock(h);
					ReleaseResource(h);
				}
			}
		}
	}
	return osErr;
}


OSErr	ConnectionSave(ConnHandle connection) {
	auto		OSErr		osErr;
	auto		Handle		h;
	auto		short		procID;
	auto		Str255		toolName;
	auto		Ptr			configStr;

	if (nil != (h = Get1Resource(kPascalText, rCMToolName))) {
		RmveResource(h);
		DisposeHandle(h);
	}

	if (nil != (h = Get1Resource(kCText, rCMConfigStr))) {
		RmveResource(h);
		DisposeHandle(h);
	}
	
	procID = (**connection).procID;
	CMGetToolName(procID, toolName);
	if (noErr == (osErr = AddResourcePtr(toolName, sizeof(toolName), kPascalText, rCMToolName, "\pCMToolName"))) {
		if (noErr == (osErr = ((nil == (configStr = CMGetConfig(connection))) ? kUnknownError : noErr))) {
			if (noErr == (osErr = AddResourcePtr(configStr, GetPtrSize(configStr), kCText, rCMConfigStr, "\pCMConfigStr"))) {
				HLock((Handle)connection);
				BlockMove((**connection).config, (**connection).oldConfig, GetPtrSize((**connection).oldConfig));
				HUnlock((Handle)connection);
				osErr = noErr;
			}
			DisposPtr(configStr);
		}
	}
	
	return osErr;
}

OSErr	ConnectionClose(ConnHandle connection) {
	auto	OSErr			osErr;
	
	if (nil != connection) {
		osErr = ConnectionDisconnect(connection);
		CMDispose(connection);
	}
	return osErr;
}

OSErr	ConnectionIdle(ConnHandle connection) {
	auto	OSErr			osErr	= noErr;
	auto	SessionPtr		session;
	auto	CMBufferSizes	sizes;
	auto	CMStatFlags		status;

	session = (SessionPtr) CMGetRefCon(connection);
	if (noErr == (osErr = CMStatus(connection, sizes, &status))) {
		if (status & cmStatusIncomingCallPresent) {
			if (noErr == (osErr = CMAccept(connection, true)))
				osErr = noErr;
		}
		if (status & cmStatusDataAvail) {
			(*session).cmCount = sizes[cmDataIn] < kBufferSize ? sizes[cmDataIn] : kBufferSize;
			if (noErr == (osErr = CMRead(connection, (*session).cmBuffer, &(*session).cmCount, cmData, false, nil, 60, &(*session).cmFlags))) {
				osErr = noErr;
			}
		} else {
			(*session).cmCount = 0;
		}
	}
	
	return osErr;
}

OSErr	ConnectionConnect(ConnHandle connection) {
	auto	OSErr			osErr	= noErr;
	
	osErr = CMOpen(connection, true, nil, -1);
	return osErr;
}

OSErr	ConnectionListen(ConnHandle connection) {
	auto	OSErr			osErr	= noErr;

	osErr = CMListen(connection, true, nil, -1);
	return osErr;
}

OSErr ConnectionDisconnect(ConnHandle connection) {
	auto	OSErr			osErr	= noErr;
	auto	CMBufferSizes	size;
	auto	CMStatFlags		status;
	
	if (noErr == (osErr = CMStatus(connection, size, &status)))
		if ((status & cmStatusOpening) || (status & cmStatusListenPend))
			osErr = CMAbort(connection);
		else if (status & cmStatusOpen)
			osErr = CMClose(connection, false, nil, 360, true);
			
	return osErr;
}
