#include <Sessions.h>

pascal long	FTSendProc(Ptr ptr, long size, SessionPtr session, CMChannel channel, short flags) {
	auto	long	retVal;
	
	retVal =  (noErr == CMWrite((*session).connection, ptr, &size, channel, false, nil, 0, flags)) ? size : 0;
	CMIdle((*session).connection);
	return retVal;
}


pascal long	FTRecvProc(Ptr ptr, long size, SessionPtr session, CMChannel channel, short *flags) {
	auto	long			retValue		= 0;
	auto	CMBufferSizes	sizes;
	auto	CMStatFlags		status;

	CMIdle((*session).connection);
	if (0 >= (*session).ftCount) {
		if (noErr == ConnectionIdle((*session).connection)) {
			(*session).ftCount		= (*session).cmCount;
			(*session).ftBuffer		= &(*session).cmBuffer[0];
		} else 
			return 0;
	}

	if (0 < (*session).ftCount) {
		while ((0 < (*session).ftCount) && (0 < size)) {
			*(ptr++) = *((*session).ftBuffer++);
			--(*session).ftCount;
			--size;
			++retValue;
		}
		*flags = (0 == (*session).ftCount ? (*session).cmFlags : 0);
	}

	return retValue;
}

pascal CMErr	FTEnvironProc(SessionPtr session, ConnEnvironRec *environ) {
	return CMGetConnEnvirons((*session).connection, environ);
}

OSErr	FileTransferNew(FTHandle *fileTransfer, WindowPtr window, long refCon) {
	auto	OSErr			osErr		= noErr;
	auto	short			procID;
	auto	Str255			toolName;

	if (noErr == (osErr = CRMGetIndToolName(classFT, 1, toolName))) { 
		osErr = kUnknownError;
		procID = FTGetProcID(toolName);
		if (-1 != procID)
			if (nil == ((*fileTransfer) = FTNew(
										procID, 
										0, 
										(ProcPtr)	&FTSendProc, 
										(ProcPtr)	&FTRecvProc, 
										(ProcPtr)	nil, 
										(ProcPtr)	nil, 
										(ProcPtr)	&FTEnvironProc, 
										(WindowPtr) window, 
										refCon, 
										0
									)))
				osErr = ResError();
			else {
				FileTransferAutoOn(*fileTransfer);
				osErr = noErr;
			}
	}
	return osErr;
}

OSErr	FileTransferOpen(FTHandle *fileTransfer, WindowPtr window, long refCon) {
	auto		OSErr			osErr		= noErr;
	register	Handle			h;
	register	short			procID;
	
	if (nil == (h = Get1Resource(kPascalText, rFTToolName)))
		osErr = ResError();
	else {
		HLock(h);
		procID = FTGetProcID(*h);
		HUnlock(h);
		ReleaseResource(h);
		if (-1 == procID)
			osErr = kUnknownError;
		else {
			if (nil == ((*fileTransfer) = FTNew(
									procID, 
									0, 
									(ProcPtr)	&FTSendProc, 
									(ProcPtr)	&FTRecvProc, 
									(ProcPtr)	nil, 
									(ProcPtr)	nil, 
									(ProcPtr)	&FTEnvironProc, 
									(WindowPtr)	window, 
									(long)		refCon, 
									0
								)))
				osErr = kUnknownError;
			else {
				if (nil == (h = Get1Resource(kCText, rFTConfigStr)))
					osErr = ResError();
				else {
					HLock(h);
					osErr = FTSetConfig(*fileTransfer, *h);
					HUnlock(h);
					ReleaseResource(h);
					if (noErr == osErr)
						FileTransferAutoOn(*fileTransfer);
				}
			}
		}
	}
	return osErr;
}

OSErr	FileTransferSave(FTHandle fileTransfer) {
	auto	OSErr			osErr		= noErr;
	auto	Handle			h;
	auto	short			procID;
	auto	Str255			toolName;
	auto	Ptr				configStr;

	if (nil != (h = Get1Resource(kPascalText, rFTToolName))) {
		RmveResource(h);
		DisposeHandle(h);
	}

	if (nil != (h = Get1Resource(kCText, rFTConfigStr))) {
		RmveResource(h);
		DisposeHandle(h);
	}

	procID = (**fileTransfer).procID;
	FTGetToolName(procID, toolName);
	if (noErr == (osErr = AddResourcePtr(toolName, sizeof(toolName), kPascalText, rFTToolName, "\pFTToolName"))) {
		if (noErr == (osErr = ((nil == (configStr = FTGetConfig(fileTransfer))) ? kUnknownError : noErr))) {
			if (noErr == (osErr = AddResourcePtr(configStr, GetPtrSize(configStr), kCText, rFTConfigStr, "\pFTConfigStr"))) {
				HLock((Handle)fileTransfer);
				BlockMove((**fileTransfer).config, (**fileTransfer).oldConfig, GetPtrSize((**fileTransfer).oldConfig));
				HUnlock((Handle)fileTransfer);
				osErr = noErr;
			}
			DisposPtr(configStr);
		}
	}
	return osErr;
}

OSErr FileTransferClose(FTHandle fileTransfer) {
	if (nil != fileTransfer) {
		FileTransferAutoOff(fileTransfer);
		FTDispose(fileTransfer);
	}
	return noErr;
}

OSErr	FileTransferIdle(FTHandle fileTransfer) {
	auto	OSErr			osErr		= noErr;
	auto	SessionPtr		session;
	auto	WDParam			pb;

	session = (SessionPtr) FTGetRefCon(fileTransfer);
	if ((*session).ftStart) {
		(*session).ftStart = false;
		(*session).ftInProgress = true;
		FileTransferAutoOff((*session).fileTransfer);
		osErr = FTStart(fileTransfer, (*session).ftDirection, &(*session).ftReply);
	} else if ((*session).ftInProgress) {
		if (ftIsFTMode & (**(*session).fileTransfer).flags) {
			FTExec((*session).fileTransfer);
		} else {
			(*session).ftInProgress = false;
			if (! (ftSucc & (**(*session).fileTransfer).flags))
				osErr = cmGenericError;
			FileTransferAutoOn((*session).fileTransfer);
			osErr = CloseWD((*session).ftReply.vRefNum);
			osErr = noErr; // aaaaaaaggggghhhhhhhhh
		}
	}
	return osErr;
}

OSErr	FileTransferSend(FTHandle fileTransfer, FSSpecPtr fsSpec) {
	auto	OSErr			osErr		= noErr;
	auto	SessionPtr		session;
	auto	short			i;
	auto	HParamBlockRec	pb;
	
	pb.wdParam.ioCompletion	= nil;
	pb.wdParam.ioNamePtr	= nil;
	pb.wdParam.ioVRefNum	= (*fsSpec).vRefNum;
	pb.wdParam.ioWDProcID	= kCreatorType;
	pb.wdParam.ioWDDirID	= (*fsSpec).parID;
	if (noErr == (osErr = PBOpenWD((WDPBPtr) &pb, false))) {
		session = (SessionPtr) FTGetRefCon(fileTransfer);
		(*session).ftReply.good		= true;
		(*session).ftReply.vRefNum	= pb.wdParam.ioVRefNum;
		for (i = 0; i < sizeof((*session).ftReply.fName); ++i)
			(*session).ftReply.fName[i]	= (*fsSpec).name[i];
		(*session).ftStart			= true;
		(*session).ftDirection		= ftTransmitting;
		(*session).ftBuffer			= (*session).cmBuffer;
	}
	return osErr;
}

OSErr	FileTransferReceive(FTHandle fileTransfer) {
	auto	OSErr			osErr		= noErr;
	auto	SessionPtr		session;

	session 					= (SessionPtr) FTGetRefCon(fileTransfer);
	(*session).ftStart			= true;
	(*session).ftDirection		= ftReceiving;
	(*session).searchPosition	= (*session).cmBuffer;

	(*session).ftReply.good		= true;
	(*session).ftReply.vRefNum	= 0;
	(*session).ftReply.fName[0]	= 0;

	return osErr;
}

pascal void	FTAutoRecCallback(ConnHandle connection, Ptr ptr, long refNum) {
	auto	SessionPtr		session;
	
	session 					= (SessionPtr) CMGetRefCon(connection);
	(*session).ftStart			= true;
	(*session).ftDirection		= ftReceiving;
	(*session).searchPosition	= ptr;
	(*session).ftBuffer			= (*session).searchPosition - (**(*session).fileTransfer).autoRec[0];

	(*session).ftReply.good		= true;
	(*session).ftReply.vRefNum	= 0;
	(*session).ftReply.fName[0]	= 0;

}

OSErr	FileTransferAutoOff(FTHandle fileTransfer) {
	auto	OSErr			osErr		= noErr;
	auto	SessionPtr		session;

	session = (SessionPtr) FTGetRefCon(fileTransfer);
	if (0 < (*session).ftSearchID) {
		CMRemoveSearch((*session).connection, (*session).ftSearchID);
		(*session).ftSearchID = 0;
	}
	return osErr;
}

OSErr	FileTransferAutoOn(FTHandle fileTransfer) {
	auto	OSErr			osErr		= noErr;
	auto	SessionPtr		session;

	if ((**fileTransfer).autoRec[0]) {
		session = (SessionPtr) FTGetRefCon(fileTransfer);
		HLock((Handle)fileTransfer);
		(*session).ftSearchID = CMAddSearch( (*session).connection, (**fileTransfer).autoRec, cmSearchSevenBit, (ProcPtr)FTAutoRecCallback);
		HUnlock((Handle)fileTransfer);
	}
	return osErr;
}

